# -*- coding: utf-8 -*-
"""
Hier even uitleg schrijven

@author: coenv
"""

#create GCODE using Python - Coen Vonk (SPC)

import math

#Pick GCODE name

f = open("CV_GNP5_PU9_859um.GCODE", "a")

#Enter ink parameters
Shear_th_index=0.396
cont_angle=88.89

#Enter printer parameters
Noz_diameter= 0.859 #in millimeters
Target_Shearrate = 100
Pulses_per_uL = round(1346.1* (Noz_diameter)**-0.257)

#Choose which head you are using
Head = 2                            #left to right, 1 through 5
Head_number = "H"+str(Head)
Target_head = "T1" + str(Head)

#=====Levels DoE=====

#Printing speed

R_lvl0 = 0.5 #printing speed needed to achieve 100s-1 of shear halfway the nozzle (R* = 0.5)
R_DoE_step = 0.10

F_5 =round(((30*Shear_th_index*Target_Shearrate*Noz_diameter)/((3*Shear_th_index)+1))*(1/(R_lvl0-(1.414*R_DoE_step))))
F_4 =round(((30*Shear_th_index*Target_Shearrate*Noz_diameter)/((3*Shear_th_index)+1))*(1/(R_lvl0-R_DoE_step)))
F_3 =round(((30*Shear_th_index*Target_Shearrate*Noz_diameter)/((3*Shear_th_index)+1))*(1/(R_lvl0)))
F_2 =round(((30*Shear_th_index*Target_Shearrate*Noz_diameter)/((3*Shear_th_index)+1))*(1/(R_lvl0+R_DoE_step)))
F_1 =round(((30*Shear_th_index*Target_Shearrate*Noz_diameter)/((3*Shear_th_index)+1))*(1/(R_lvl0+(1.414*R_DoE_step))))

F_list = [F_2, F_1, F_2, F_3, F_4, F_5, F_3, F_3, F_3, F_4]

#Nozzle substrate distance

NSD_lvl0 = 1 #optimal spacing
NSD_DoE_step = 0.15

cont_angle_r=cont_angle*(math.pi/180) #convert to radians

NSD_1 = round(((1-math.cos(cont_angle_r))*math.sqrt(math.pi))/(math.sqrt( ((4)*cont_angle_r)) - (2*(math.sin(2*cont_angle_r))))*Noz_diameter*(NSD_lvl0-(1.414*NSD_DoE_step)),3)
NSD_2 = round(((1-math.cos(cont_angle_r))*math.sqrt(math.pi))/(math.sqrt( ((4)*cont_angle_r)) - (2*(math.sin(2*cont_angle_r))))*Noz_diameter*(NSD_lvl0-NSD_DoE_step),3)
NSD_3 = round(((1-math.cos(cont_angle_r))*math.sqrt(math.pi))/(math.sqrt( ((4)*cont_angle_r)) - (2*(math.sin(2*cont_angle_r))))*Noz_diameter*(1),3)
NSD_4 = round(((1-math.cos(cont_angle_r))*math.sqrt(math.pi))/(math.sqrt( ((4)*cont_angle_r)) - (2*(math.sin(2*cont_angle_r))))*Noz_diameter*(NSD_lvl0+NSD_DoE_step),3)
NSD_5 = round(((1-math.cos(cont_angle_r))*math.sqrt(math.pi))/(math.sqrt( ((4)*cont_angle_r)) - (2*(math.sin(2*cont_angle_r))))*Noz_diameter*(NSD_lvl0+(1.414*NSD_DoE_step)),3)

NSD_list = [NSD_2, NSD_3, NSD_4, NSD_5, NSD_4, NSD_3, NSD_1, NSD_3, NSD_3, NSD_2]

print(";+---------------+-------+-------+-------+-------+-------+", file=f)
print(";|               |-1.414 |   -1  |   0   |   1   | 1.414 |", file=f) 
print(";+---------------+-------+-------+-------+-------+-------+", file=f)
print(";|   F [mm/min]  |  "+str(F_1)+"  |  "+str(F_2)+"  |  "+str(F_3)+"  |  "+str(F_4)+"  |  "+str(F_5)+"  |", file=f)
print(";+---------------+-------+-------+-------+-------+-------+", file=f)
print(";|  NSD [mm]     | "+str(NSD_1)+" | "+str(NSD_2)+" | "+str(NSD_3)+" | "+str(NSD_4)+" | "+str(NSD_5)+" |", file=f)
print(";+---------------+-------+-------+-------+-------+-------+", file=f)

# Home all axes

print("G28 Z0", file=f)
print("G28 X0 Y0", file=f)

#Choose starting Z position
Z_start = " Z100"

print("M660 "+Head_number+Z_start+" ; Define starting Z position", file=f)

#define starting X and Y

X_start = "X100 "
Y_start = "Y50"

print("M6 "+Target_head+" O2 "+X_start+Y_start+" ; Define starting X and Y position", file=f)

#Choose printing temperature
#T_h = "[temperature_head]"
#print("M109 T12 S"+T_h)

#Choose tempeature for bed
#T_b = "[temperature_bed]"
#print("M190 S"+T_b)

#Set priming parameters

Prim_speed_n = "S10000" #speed at which prime move should be executed (usually 10000)
Prim_pulses_n = "E1000" #number of pulses on the feed to execute
Prim_dwelltime_n = "P2000" #number of milliseconds to dwell relative to start of the move

print("M722 "+Prim_speed_n+" "+Prim_pulses_n+" "+Prim_dwelltime_n+" "+Target_head+" ; Set priming parameters", file=f)

#Set unpriming parameters

unPrim_speed_n = "S10000" #speed at which unprime move should be executed (usually 10000)
unPrim_pulses_n = "E1000" #number of pulses on the feed to execute
unPrim_dwelltime_n = "P-2000" #number of milliseconds to dwell relative to start of the move (usually negative number!)

print("M721 "+unPrim_speed_n+" "+unPrim_pulses_n+" "+unPrim_dwelltime_n+" "+Target_head+" ; Set unpriming parameters", file=f)

#Set flow rate

Flow_P_n = "P"+str(Pulses_per_uL) #Number of pulses on the motor to dispense 1 uL of material
Flow_S_n = "S"+str(0.785) #Direct flow multiplier
Flow_W_n = "W"+ str(Noz_diameter) #Width of the cross section of the volume to fill
Flow_Z_n = "Z"+ str(Noz_diameter) #Height of the cross section of the volume to fill

print("M221 "+Flow_P_n+" "+Flow_S_n+" "+Target_head+" "+Flow_Z_n+" "+Flow_W_n+" ; Set flow parameters", file=f)

#Set units to milimeters
print("G21", file=f)

#Set coordinates: relative
print("G91", file=f)
 
#Set coordinates: absolute
print("G90", file=f)

#Use absolute distances for extrusion
print("M82", file=f)

#Unknown
print("G92 E0", file=f)

#Invoke starting X and Y
print("T"+str((Head-1)), file=f)

#START PRINTING
print(";-----Start printing-----", file=f)

print("M221 "+Target_head+" "+Flow_P_n, file=f) #set new pulses per uL
print("G0 X0 Y0", file=f) #goes to starting point XY
print("G0 Z"+str(round(NSD_3,3))+" "+Head_number, file=f) #goes to starting point Z
print("M221 "+Target_head+" "+Flow_S_n, file=f) #set correction factor for circular nozzle

Y_1 = 0
Y_2 = 3
Y_3 = 5

for i in range(1,6):
    #calibration region of print 1
    print("G0 Z"+str(NSD_3)+" "+Head_number, file=f)           #Anchor Point
    print("G1 X0 Y"+str(Y_2)+" E1 F"+str(round(F_3)), file=f)           #Anchor Point

    print("G1 X2 Y"+str(Y_2)+" E1", file=f)
    print("G1 X2 Y"+str(Y_1)+" E1", file=f)
    print("G1 X4 Y"+str(Y_1)+" E1", file=f)
    print("G1 X4 Y"+str(Y_3)+" E1", file=f)
    print("G1 X6 Y"+str(Y_3)+" E1", file=f)
    print("G1 X6 Y"+str(Y_1)+" E1", file=f)
    print("G1 X8 Y"+str(Y_1)+" E1", file=f)
    print("G1 X8 Y"+str(Y_3)+" E1", file=f)
    print("G1 X10 Y"+str(Y_3)+" E1", file=f)
    print("G1 X10 Y"+str(Y_1)+" E1", file=f)
    print("G1 X12 Y"+str(Y_1)+" E1", file=f)
    print("G1 X12 Y"+str(Y_2)+" E1", file=f)

    #conductivity region of print 1

    print("G0 Z"+str(NSD_list[2*i-2])+" "+Head_number, file=f)                    #Anchor Point
    print("G1 X36 Y"+str(Y_2)+" E1 F"+str(F_list[2*i-2]), file=f)                 #Anchor Point

    #morphology region of print 1

    print("G1 X36 Y"+str(Y_3)+" E1", file=f)
    print("G1 X38 Y"+str(Y_3)+" E1", file=f)
    
    print("G1 X38 Y"+str(Y_1)+" E1", file=f)
    print("G1 X39 Y"+str(Y_1)+" E1", file=f)       #1.0mm spaced
    print("G1 X39 Y"+str(Y_3)+" E1", file=f)       #1.0mm spaced
    print("G1 X40 Y"+str(Y_3)+" E1", file=f)
    
    print("G1 X40 Y"+str(Y_1)+" E1", file=f)
    print("G1 X40.9 Y"+str(Y_1)+" E1", file=f)     #0.9mm spaced
    print("G1 X40.9 Y"+str(Y_3)+" E1", file=f)     #0.9mm spaced
    print("G1 X42 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X42 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X42.8 Y"+str(Y_1)+" E1", file=f)     #0.8mm spaced
    # print("G1 X42.8 Y"+str(Y_3)+" E1", file=f)     #0.8mm spaced
    # print("G1 X44 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X44 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X44.7 Y"+str(Y_1)+" E1", file=f)     #0.7 mm spaced
    # print("G1 X44.7 Y"+str(Y_3)+" E1", file=f)     #0.7 mm spaced
    # print("G1 X46 Y"+str(Y_3)+" E1", file=f)
    
    
    # print("G1 X46 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X46.6 Y"+str(Y_1)+" E1", file=f)     #0.6mm spaced
    # print("G1 X46.6 Y"+str(Y_3)+" E1", file=f)     #0.6mm spaced
    # print("G1 X48 Y"+str(Y_3)+" E1", file=f)
    
    
    # print("G1 X48 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X48.5 Y"+str(Y_1)+" E1", file=f)     #0.5mm spaced
    # print("G1 X48.5 Y"+str(Y_3)+" E1", file=f)     #0.5mm spaced
    # print("G1 X50 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X50 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X50.4 Y"+str(Y_1)+" E1", file=f)     #0.4mm spaced
    # print("G1 X50.4 Y"+str(Y_3)+" E1", file=f)     #0.4mm spaced
    # print("G1 X52 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X52 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X52.3 Y"+str(Y_1)+" E1", file=f)     #0.3mm spaced
    # print("G1 X52.3 Y"+str(Y_3)+" E1", file=f)     #0.3mm spaced
    # print("G1 X54 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X54 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X54.2 Y"+str(Y_1)+" E1", file=f)     #0.2mm spaced
    # print("G1 X54.2 Y"+str(Y_3)+" E1", file=f)     #0.2mm spaced
    # print("G1 X56 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X56 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X56.1 Y"+str(Y_1)+" E1", file=f)     #0.1mm spaced
    # print("G1 X56.1 Y"+str(Y_3)+" E1", file=f)     #0.1mm spaced
    
    
    print("G1 X58 Y"+str(Y_3)+" E1", file=f)
    print("G1 X58 Y"+str(Y_2)+" E1", file=f)                            #Anchor Point
    print("G1 X60 Y"+str(Y_2)+" E1", file=f)                            #Anchor Point

    #Setting new Y values for print 2
    Y_1 = Y_1 + 6
    Y_2 = Y_2 + 6
    Y_3 = Y_3 + 6

    #calibration region of print 2
    print("G0 Z"+str(NSD_3)+" "+Head_number, file=f)                    #Anchor Point
    print("G1 X60 Y"+str(Y_2)+"E1 F"+str(round(F_3)), file=f)           #Anchor Point

    print("G1 X58 Y"+str(Y_2)+" E1", file=f)
    print("G1 X58 Y"+str(Y_1)+" E1", file=f)
    print("G1 X56 Y"+str(Y_1)+" E1", file=f)
    print("G1 X56 Y"+str(Y_3)+" E1", file=f)
    print("G1 X54 Y"+str(Y_3)+" E1", file=f)
    print("G1 X54 Y"+str(Y_1)+" E1", file=f)
    print("G1 X52 Y"+str(Y_1)+" E1", file=f)
    print("G1 X52 Y"+str(Y_3)+" E1", file=f)
    print("G1 X50 Y"+str(Y_3)+" E1", file=f)
    print("G1 X50 Y"+str(Y_1)+" E1", file=f)
    print("G1 X48 Y"+str(Y_1)+" E1", file=f)
    print("G1 X48 Y"+str(Y_2)+" E1", file=f)

    #conductivity region of print 2

    print("G0 Z"+str(NSD_list[2*i-1])+" "+Head_number, file=f)                    #Anchor Point
    print("G1 X24 Y"+str(Y_2)+" E1 F"+str(F_list[2*i-1]), file=f)                 #Anchor Point

    #morphology region of print 2
    print("G1 X24 Y"+str(Y_3)+" E1", file=f)
    print("G1 X22 Y"+str(Y_3)+" E1", file=f)
    
    print("G1 X22 Y"+str(Y_1)+" E1", file=f)
    print("G1 X21 Y"+str(Y_1)+" E1", file=f)       #1.0 mm spaced
    print("G1 X21 Y"+str(Y_3)+" E1", file=f)      #1.0 mm spaced
    print("G1 X20 Y"+str(Y_3)+" E1", file=f)
    
    print("G1 X20 Y"+str(Y_1)+" E1", file=f)
    print("G1 X19.1 Y"+str(Y_1)+" E1", file=f)     #0.9mm spaced
    print("G1 X19.1 Y"+str(Y_3)+" E1", file=f)    #0.9mm spaced
    print("G1 X18 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X18 Y"+str(Y_1)+" E1", file=f) 
    # print("G1 X17.2 Y"+str(Y_1)+" E1", file=f)     #0.8mm spaced
    # print("G1 X17.2 Y"+str(Y_3)+" E1", file=f)    #0.8mm spaced
    # print("G1 X16 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X16 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X15.3 Y"+str(Y_1)+" E1", file=f)     #0.7mm spaced
    # print("G1 X15.3 Y"+str(Y_3)+" E1", file=f)    #0.7mm spaced
    # print("G1 X14 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X14 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X13.4 Y"+str(Y_1)+" E1", file=f)     #0.6mm spaced
    # print("G1 X13.4 Y"+str(Y_3)+" E1", file=f)    #0.6mm spaced
    # print("G1 X12 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X12 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X11.5 Y"+str(Y_1)+" E1", file=f)     #0.5mm spaced
    # print("G1 X11.5 Y"+str(Y_3)+" E1", file=f)    #0.5mm spaced
    # print("G1 X10 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X10 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X9.6 Y"+str(Y_1)+" E1", file=f)      #0.4mm spaced
    # print("G1 X9.6 Y"+str(Y_3)+" E1", file=f)     #0.4mm spaced
    # print("G1 X8 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X8 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X7.7 Y"+str(Y_1)+" E1", file=f)      #0.3 mm spaced
    # print("G1 X7.7 Y"+str(Y_3)+" E1", file=f)     #0.3 mm spaced
    # print("G1 X6 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X6 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X5.8 Y"+str(Y_1)+" E1", file=f)      #0.2mm spaced
    # print("G1 X5.8 Y"+str(Y_3)+" E1", file=f)     #0.2 mm spaced
    # print("G1 X4 Y"+str(Y_3)+" E1", file=f)
    
    # print("G1 X4 Y"+str(Y_1)+" E1", file=f)
    # print("G1 X3.9 Y"+str(Y_1)+" E1", file=f)      #0.1mm spaced
    # print("G1 X3.9 Y"+str(Y_3)+" E1", file=f)     #0.1mm spaced
    
    print("G1 X2 Y"+str(Y_3)+" E1", file=f)
    print("G1 X2 Y"+str(Y_2)+" E1", file=f)                             #Anchor Point
    print("G1 X0 Y"+str(Y_2)+" E1", file=f)                             #Anchor Point

    #Setting new Y values for print 3

    Y_1 = Y_1 + 6
    Y_2 = Y_2 + 6
    Y_3 = Y_3 + 6
    
#Go to park position

print("G1 X0 Y70 E1", file=f)

#Home Z-axis

print("G28 Z0", file=f)

#Clear offsets

print("G53", file=f)

#End print job

print("M30", file=f)
f.close()

print("+---------------+-------+-------+-------+-------+-------+")
print("|               |-1.414 |   -1  |   0   |   1   | 1.414 |") 
print("+---------------+-------+-------+-------+-------+-------+")
print("|   F [mm/min]  |  "+str(F_1)+"  |  "+str(F_2)+"  |  "+str(F_3)+"  |  "+str(F_4)+"  |  "+str(F_5)+"  |")
print("+---------------+-------+-------+-------+-------+-------+")
print("|  NSD [mm]     | "+str(NSD_1)+" | "+str(NSD_2)+" | "+str(NSD_3)+" | "+str(NSD_4)+" | "+str(NSD_5)+" |")
print("+---------------+-------+-------+-------+-------+-------+")